package com.slideme.demo.model.slidelock;

import com.slideme.demo.model.C;

import java.io.*;

/**
 * Basic utilities to work with SlideME SlideLock 2 library.
 * This class provides helper methods to store & retrieve a license on
 * the filesystem, along with a basic storage expiry mechanism in order
 * to enable refreshing the license from time to time
 */
public class LicenseUtils {

    private static final long LICENSE_EXPIRY = C.Time.Unit.MINUTE * 5;/* 5 minute(s)*/

    private LicenseUtils() {
        throw new Error("Not allowed.");
    }

    /**
     * @param root            root directory where licenses are stored
     * @param licenseFileName license file name
     * @return binary license data if a license is stored under this fileName
     * @throws IOException when IO Errors are encountered
     */
    public static byte[] retrieveLicense(File root, String licenseFileName) throws IOException {
        assertRoot(root);
        return retrieveLicense(new File(root, licenseFileName));
    }

    /**
     * @param aLicenseFileURI absolute license file URI
     * @return binary license data if a license is stored under this file
     * @throws IOException when IO Errors are encountered
     */
    public static byte[] retrieveLicense(String aLicenseFileURI) throws IOException {
        return retrieveLicense(new File(aLicenseFileURI));
    }

    /**
     * @param aLicenseFile file containing a binary license stored by any of the LicenseUtils#storeLicense() methods
     * @return binary license data if a license is stored under this file
     * @throws IOException when IO Errors are encountered
     *                     Note: If the license has expired it will be deleted and the return value will be null
     */
    public static byte[] retrieveLicense(File aLicenseFile) throws IOException {
        DataInputStream dis = new DataInputStream(new FileInputStream(aLicenseFile));
        long created = dis.readLong();
        long now = System.currentTimeMillis();
        if (now < created || now - LICENSE_EXPIRY > created) {/*Delete on expiry*/
            System.out.println("License expired; Attempting to delete");
            aLicenseFile.delete();
            return null;
        }

        int contentSize = dis.readInt();
        byte[] data = new byte[contentSize];
        dis.readFully(data);
        try {
            dis.close();
        } catch (IOException ignored) {
        }
        return data;
    }

    /**
     * @param root            root directory where licenses are stored
     * @param licenseFileName license file name
     * @param licenseData     license binary value
     * @throws IOException when IO Errors are encountered
     */
    public static void storeLicense(File root, String licenseFileName, byte[] licenseData) throws IOException {
        assertRoot(root);
        storeLicense(new File(root, licenseFileName), licenseData);
    }

    /**
     * @param licenseFileURI absolute license file URI
     * @param licenseData    license binary value
     * @throws IOException when IO Errors are encountered
     */
    public static void storeLicense(String licenseFileURI, byte[] licenseData) throws IOException {
        storeLicense(new File(licenseFileURI), licenseData);
    }

    /**
     * @param licenseFile license file
     * @param licenseData license binary value     `
     * @throws IOException when IO Errors are encountered
     */
    public static void storeLicense(File licenseFile, byte[] licenseData) throws IOException {
        DataOutputStream dos = new DataOutputStream(new FileOutputStream(licenseFile, false));
        dos.writeLong(System.currentTimeMillis());  /* Store current timestamp for expiry detection */
        dos.writeInt(licenseData.length);
        dos.write(licenseData);
        dos.flush();
        try {
            dos.close();
        } catch (IOException ignored) {
        }
    }


    private static void assertRoot(File root) {
        if (root == null || !root.exists() || !root.isDirectory() || !root.canWrite())
            throw new IllegalArgumentException("Invalid root");
    }


}
